/*
   ┏━━━┓╱╱╱╱╱┏┓╱╱┏┓╱╱╱╱╱┏┓╱╱
   ┃┏━┓┃╱╱╱╱╱┃┃╱╱┃┃╱╱╱╱╱┃┃╱╱╱
   ┃┗━┛┣┓┏┳━━┫┃┏┓┃┃╱╱┏━━┫┗━┳━━┓
   ┃┏━━┫┃┃┃┏┓┃┗┛┃┃┃╱┏┫┏┓┃┏┓┃━━┫
   ┃┃╱╱┃┗┛┃┃┃┃┏┓┃┃┗━┛┃┏┓┃┗┛┣━━┃
   ┗┛╱╱┗━━┻┛┗┻┛┗┛┗━━━┻┛┗┻━━┻━━┛
    ━━━━━━━━━━━━━━━━━━━━━━━━━━

    Copyright (c) 2022 Punk Labs LLC

    This section is part of OneTrick

    OneTrick is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation, either version 3 of the License, or (at your option)
    any later version.

    OneTrick is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    OneTrick.  If not, see <http://www.gnu.org/licenses/>.
*/

use nih_plug_egui::egui::{
    Painter,
    Pos2,
    Vec2,
    Rect,
    Rounding,
    Color32,
};


pub trait TextEffectDrawer {
    fn fade_80s(&self, rect: Rect, count: u32, color: Color32);
    fn fade_80s_uniform(&self, rect: Rect, spacing: f32, color: Color32);
}

impl TextEffectDrawer for Painter {
    fn fade_80s(&self, rect: Rect, count: u32, color: Color32) {
        //self.rect_filled(rect, Rounding::none(), Color32::from_white_alpha(64));
        let spacing = rect.height() / (count as f32);
        let smallest = 1.0 / (count as f32);
        let start_y = rect.top() + spacing * 0.5 - smallest;
        let start_x = rect.left();
        let width = rect.width();
        for i in 0..count {
            let height = ((i+1) as f32) / (count as f32) * spacing;
            let line_rect = Rect::from_min_size(Pos2::new(start_x, start_y+spacing*(i as f32)), Vec2::new(width, height));
            self.rect_filled(
                line_rect,
                Rounding::none(),
                color,
            );
        }
    }
    fn fade_80s_uniform(&self, rect: Rect, spacing: f32, color: Color32) {
        //self.rect_filled(rect, Rounding::none(), Color32::from_white_alpha(64));
        let start_y = rect.top();
        let count = (rect.height() / spacing * 0.5) as u32;
        let start_x = rect.left();
        let width = rect.width();
        for i in 0..count {
            let line_rect = Rect::from_min_size(Pos2::new(start_x, start_y+spacing*2.0*(i as f32)), Vec2::new(width, spacing));
            let a = (((i+1) as f32 / count as f32) * color.a() as f32) as u8;
            let color = Color32::from_rgba_unmultiplied(color.r(), color.g(), color.b(), a);
            self.rect_filled(
                line_rect,
                Rounding::none(),
                color,
            );
        }
    }
}

